import requests
from datetime import datetime
from requests.exceptions import Timeout



def getchannelvideocount(username,proxy=None):
    response=None
    url = "https://www.aparat.com/api/fa/v1/user/user/information/username/" + username
    try:
        if proxy==None:
            response = requests.request("GET", url, proxies=proxy, timeout=10)
        else:
            response = requests.request("GET", url, timeout=10)
    except Timeout:
        print("The request timed out after 10 seconds.")
    except requests.RequestException as e:
        print(f"An error occurred: {e}")

    if response.status_code == 404:
        return 404
    data = response.json()
    video_cnt=int(data["data"]["attributes"]["video_cnt"])

    return video_cnt


def get_video_commnets(video_uid,proxy=None):
    comment_list=[]
    url = "https://www.aparat.com/api/fa/v1/video/comment/list/videohash/"+video_uid
    querystring = {"perpage":"999999999999999"}
    payload = ""
    response=""
    
    if proxy != None:
        response = requests.request("GET", url, data=payload,proxies=proxies[0], timeout=timeout_seconds)
    else:
        response = requests.request("GET", url, timeout=10)
    data=response.json()
    
    if data["data"]:
       pass
    else:
        #print("no commnet")
        return {"uid":video_uid ,"data":None}
    comment_list=[]
    for comment in data["data"]:
        userid=comment["relationships"]["channel"]["data"]["id"]
        comment_owner_username=""
        comment_owner_name=""
        username_list=[]

        for item in data["included"]:
            if item.get("id") == userid:
                found_object = item
                comment_owner_username=found_object["attributes"]["username"]
                comment_owner_name=found_object["attributes"]["name"]
                comment_owner_avatar=found_object["attributes"]["avatar"]
                break

        cmnt={
            "uid": video_uid,
            "id":  comment["attributes"]["id"] if "id" in comment["attributes"] else None,
            "body": comment["attributes"]["body"] if "body" in comment["attributes"]else None,
            "reply": comment["attributes"]["reply"] if "reply" in comment["attributes"]else None,
            "date_gregorian": comment["attributes"]["sdate_gregorian"] if "sdate_gregorian" in comment["attributes"]else None,
            "timestamp": int(datetime.strptime(comment["attributes"]["sdate_gregorian"], "%Y-%m-%d %H:%M:%S").timestamp()) if "sdate_gregorian" in comment["attributes"]else None,
            "like_count": int(comment["attributes"]["like_cnt"]) if "like_cnt" in comment["attributes"] else None,
            "reply_count": int(comment["attributes"]["reply_cnt"]) if "reply_cnt" in comment["attributes"] else None,
            "owner_id": userid,
            "owner_username": comment_owner_username,
            "owner_name": comment_owner_name,
            "owner_avatar": comment_owner_avatar
        }
        comment_list.append(cmnt)

    data={"uid":video_uid,
                "data":comment_list}

    return data




def getvideos(usrname,video_count=None,proxy=None):
    if not video_count:
        video_count=getchannelvideocount(usrname,proxy)
        
    maxpage=(video_count/40)+1
    pagenumber=0
    payload = ""
    headers = {"cookie": "playIconOnHover_1=new; AFCN=169320560431281; apr_lb_id=m25","User-Agent": "firefox/2023.5.3"}
    first_id=''
    videos=[]
    while True:
        pagenumber+=1
        print(str(pagenumber) +" of "+str(maxpage))
        if pagenumber>=maxpage:
            break
        url = f"https://www.aparat.com/api/fa/v1/user/video/list/username/{usrname}/page/{pagenumber}/perpage/40/isnextpage/true"
        response = requests.request("GET", url, data=payload, headers=headers, timeout=10)
        if response.status_code==429:
            print("faild with server ip 429 error try ing with proxy")
            response = requests.request("GET", url, data=payload, headers=headers,proxies=random.choice(proxies), timeout=timeout_seconds)
            print(response.status_code)
        if response.status_code != 200:
            msg="status code is "+str(response.status_code)
            sentry_sdk.capture_message("msg", level="error")
            print(msg)
            if response.status_code == 404:
                break
        data=response.json()


        total=data["data"][0]["attributes"]["total"]
        if total==0:
            break

        includeds=data["included"]

        for included in includeds:
            if "uid" in included["attributes"]:
                UID=included["attributes"]["uid"]  
            else:
                break
            

            if "id" not in included:
                break
            if pagenumber==1:
                first_id=included["id"]
            if pagenumber>2:
                first_id==included["id"]
                print("WE ARE IN A LOOP")
                pagenumber=999999999999999999999
            #print(included)
            # print("********************")
            # print(included["attributes"]["visit_cnt_int"])
            # print(type(included["attributes"]["visit_cnt_int"]))
            # print("********************")
            comment_data=get_video_commnets(included["attributes"]["uid"],proxy=None)

            video={
                "platform": "Aparat",
                "_":"video",
                "id":included["id"] if "id" in included else None,
                "owner_username":included["attributes"]["username"] if "username" in included["attributes"] else None,
                "owner_id":int(included["attributes"]["userid"]) if "userid" in included["attributes"] else 0,
                "title":included["attributes"]["title"] if "title" in included["attributes"] else None,
                "tags":included["attributes"]["tags"] if "tags" in included["attributes"] else None,
                "uid":included["attributes"]["uid"] if "uid" in included["attributes"] else None,
                "visit_count":int(included["attributes"]["visit_cnt_int"]) if "visit_cnt_int" in included["attributes"] and included["attributes"]["visit_cnt_int"].isdigit() else 0,
                "owner_name":included["attributes"]["sender_name"] if "sender_name" in included["attributes"] else None,
                "poster":included["attributes"]["big_poster"] if "big_poster" in included["attributes"] else None,
                "owner_avatar":included["attributes"]["profilePhoto"] if "profilePhoto" in included["attributes"] else None,
                "duration":int(included["attributes"]["duration"]) if "duration" in included["attributes"] else 0,
                "posted_date":included["attributes"]["sdate_rss"] ,
                "posted_timestamp":int(datetime.strptime(included["attributes"]["sdate_rss"], "%Y-%m-%d %H:%M:%S").timestamp()) ,#2015-05-17 12:12:51
                
                "sdate_rss":included["attributes"]["sdate_rss"] ,
                "sdate_rss_tp":int(datetime.strptime(included["attributes"]["sdate_rss"], "%Y-%m-%d %H:%M:%S").timestamp()) ,#2015-05-17 12:12:51
                "comments":comment_data["data"],
                "frame":included["attributes"]["frame"] if "frame" in included["attributes"] else None,
                "like_count":int(included["attributes"]["like_cnt"]) if "like_cnt" in included["attributes"] and included["attributes"]["like_cnt"] != None else 0,
                "description":included["attributes"]["description"] if "description" in included["attributes"]  else None,
                "is_deleted": False,

            }
            videos.append(video)

    return videos



import requests


def get_video_recom(videouid,proxy=None):
    
    url = "https://www.aparat.com/api/fa/v1/video/recom/list_v2/videohash/"+videouid
    response=None
    timeout_seconds=10
    
    if proxy!=None:
        response = requests.request("GET", url, timeout=timeout_seconds)
    else:
        response = requests.request("GET", url, proxies=proxy, timeout=timeout_seconds)

    json_responce=response.json()
    attribute=json_responce["data"]["attributes"]["video_recom"]
    channels=[]
    videos=[]
    for attribute in json_responce["data"]["attributes"]["video_recom"]:
        channels.append(attribute["username"])
        videos.append(attribute["uid"])

            
    return {"videos_uids":videos,"channels":list(set(channels))}


import requests
from bs4 import BeautifulSoup


official_channel_pages=['https://www.aparat.com/official','https://www.aparat.com/official/real']


def  OfficialChannelsUsernames(urllist=official_channel_pages):
    details_divs=[]
    for url in urllist:
        response = requests.get(url)
        html_content = response.content
        detlis=BeautifulSoup(html_content, 'html.parser').find_all('div', class_='details')
        for detli in detlis:
            details_divs.append(detli.find('a', class_='name').get('href').replace("/",''))
    details_divs=list(set(details_divs))
    return details_divs





import requests
from requests.exceptions import Timeout
from persian import convert_ar_characters
from datetime import datetime



def convert_numerical_string_to_int(numerical_string):
    converted = convert_ar_characters(numerical_string)
    if "میلیارد" in converted:
        return int(float(converted.replace(" میلیارد", "")) * 1000000000)
    if "میلیون" in converted:
        return int(float(converted.replace(" میلیون", "")) * 1000000)
    elif "هزار" in converted:
        return int(float(converted.replace(" هزار", "")) * 1000)
    else:
        data = ""
        try:
            data = int(converted)
        except:
            data = ""

        return data



def channelDetail(username,proxy=None):
    global response
    response=None
    url = "https://www.aparat.com/api/fa/v1/user/user/information/username/" + username
    try:
        if proxy==None:
            response = requests.request("GET", url, proxies=proxy, timeout=10)
        else:
            response = requests.request("GET", url, timeout=10)
    except Timeout:
        print("The request timed out after 10 seconds.")
    except requests.RequestException as e:
        print(f"An error occurred: {e}")

    if response.status_code == 404:
        return 404
    data = response.json()
    data=data["data"]["attributes"]

    new_channel = {
        "_":"channel",
        "username": data["username"] if "username" in data else None,
        "userid": int(data["id"]) if "id" in data and data["id"] is not None else None,
        "avatar_thumbnail": data["pic_m"] if "pic_m" in data else None,
        "is_official": True if 'official' in data and data['official'] is not None else False,
        "name": data["name"] if "name" in data else None,
        "bio_links": [data["url"]] if "url" in data and data["url"] != '' else None,
        "description": data["description"] if "description" in data else None,
        "total_video_visit": int(convert_numerical_string_to_int(data["video_visit"])) if "video_visit" in data and data["video_visit"]!="" else 0,
        "video_count": int(data["video_cnt"]) if "video_cnt" in data and data["video_cnt"] is not None else None,
        "start_date": data["start_date"] if data["start_date"] else None,
        "start_date_timestamp": int(datetime.strptime(data["start_date"], "%Y-%m-%d").timestamp()),
        "followers_count":int(convert_numerical_string_to_int(data["follower_cnt"])) if "follower_cnt" in data and data["follower_cnt"] not in [None, ""] else None,
        "following_count": int(convert_numerical_string_to_int(data["follow_cnt"])) if "follow_cnt" in data and data["follow_cnt"] not in [None, ""] else None,
        "is_deleted": False,
        "country": "iran",
        "platform": "aparat"
    }

    return new_channel


from setuptools import setup, find_packages

setup(
    name='aparat_crawler',
    version='0.3',
    packages=find_packages(),
    author='Mohammad Amin Orojloo',
    author_email='ma.orojloo@gmail.com',
    description='this is an aparat crawler library',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/maorojloo/aparat_crawler',
    install_requires=[
        'beautifulsoup4==4.12.3',
        'persian==0.5.0',
        'requests==2.31.0',
    ],
    classifiers=[
        'Programming Language :: Python :: 3.10',
    ],
)



import requests
from datetime import datetime
timeout_seconds=10



nub=0
included=[]
def fetchSearchResult(url,proxy=None):
    global nub
    global included

    if proxy==None:
        response = requests.request("GET", url,proxies=proxy, timeout=timeout_seconds)
    else:
        response = requests.request("GET", url, timeout=timeout_seconds)

    json_responce=response.json()
    if json_responce["data"][0]["attributes"]["link"] != None:
        nextpage=json_responce["data"][0]["attributes"]["link"]["next"]
        
        
        
        nub+=1
        print(nub)
        if nextpage != None and nextpage!="":
            included+=json_responce["included"]
            fetchSearchResult(nextpage)
    else:
        print(f"fetching done in {nub} pages.")
        print(type(included))
    
    return included


def searchByKeyword(keyword,uploadedIn=None,proxy=None):
    timeout_seconds=10

    if len(keyword)<3:
        raise ValueError(f"keyword input: must me in range of 3 to 512 character")

    valid_uploadedIn=["today","year","week",None]
    if uploadedIn not in valid_uploadedIn:
        raise ValueError(f"Invalid input: {uploadedIn}. Expected one of {valid_uploadedIn}")
    url=None
    if uploadedIn==None:
        url=f"https://www.aparat.com/api/fa/v1/video/video/search/text/{keyword}/?type_search=search"
    else:
        url=f"https://www.aparat.com/api/fa/v1/video/video/search/text/{keyword}/?type_search=search&uploadedIn={uploadedIn}"


    raw_results=fetchSearchResult(url)

    uids=[]
    channels=[]
    for raw_result in raw_results:
        if raw_result["type"]=="channel":
            channels.append(raw_result["attributes"]["username"])
             


        if raw_result["type"]=="Video":
            uids.append(raw_result["attributes"]["uid"])


    

    return {"raw_results":raw_results,"videos_uids":uids,"channels":channels}




import requests
from requests.exceptions import Timeout

proxy_timeout =10



uids=[]
channels=[]
def fetchFirstPage(proxy=None):

    
    for pagenumber in range(0,100):
        print(pagenumber)
        
        url=f"https://www.aparat.com/api/fa/v1/video/video/list/tagid/1?page={pagenumber}"
        response=None
        included=[]
        try:
            if proxy==None:
                response = requests.request("GET", url, proxies=proxy, timeout=proxy_timeout)
            else:
                response = requests.request("GET", url, timeout=proxy_timeout)
            json_responce=response.json()
            included+=json_responce["included"]
        except Timeout:
            print(f"The request timed out after {proxy_timeout} seconds.")
        except requests.RequestException as e:
            print(f"An error occurred: {e}")


        for includeditem in included:
            if includeditem["type"]=="channel":
                channels.append(includeditem["attributes"]["username"])
                
            if includeditem["type"]=="Video":
                uids.append(includeditem["attributes"]["uid"])

    return {"videos_uids":uids,"channels":list(set(channels))}


    
        
    
    

